// WARNING: This file is auto-generated!  Do not hand-edit!
// Instead, edit the GLSL shaders and run 'unix-build nvmake @generate'.

#ifndef __G_SHADER_NAMES_H__
#define __G_SHADER_NAMES_H__

typedef enum {
    PROGRAM_NVIDIA_HEADSURFACE_VERTEX,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OVERLAY,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OVERLAY_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_OVERLAY,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_OVERLAY_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_OVERLAY_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OFFSET_OVERLAY_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OVERLAY,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OVERLAY_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OVERLAY_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OVERLAY_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_OVERLAY,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_OVERLAY_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_OVERLAY_SWAPPED,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_BLEND_OFFSET_OVERLAY_SWAPPED_CUSTOMSAMPLING,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_YUV420,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_YUV420_OVERLAY,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_PIXELSHIFT,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_OVERLAY_PIXELSHIFT,
    PROGRAM_NVIDIA_HEADSURFACE_FRAGMENT_REVERSEPRIME,
} ProgramName;

#define NUM_PROGRAMS 34

#endif // __G_SHADER_NAMES_H__
